@@ -15,7 +15,6 @@ from openstack import resource
 
 
 class Chassis(_common.ListMixin, resource.Resource):
-
     resources_key = 'chassis'
     base_path = '/chassis'
 
@@ -33,7 +32,7 @@ class Chassis(_common.ListMixin, resource.Resource):
     commit_jsonpatch = True
 
     _query_mapping = resource.QueryParameters(
-        fields={'name': 'fields', 'type': _common.comma_separated_list},
+        fields={'type': _common.fields_type},
     )
 
     #: Timestamp at which the chassis was created.
