@@ -10,9 +10,13 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import typing as ty
+
 from openstack.block_storage import _base_proxy
+from openstack.block_storage.v3 import attachment as _attachment
 from openstack.block_storage.v3 import availability_zone
 from openstack.block_storage.v3 import backup as _backup
+from openstack.block_storage.v3 import block_storage_summary as _summary
 from openstack.block_storage.v3 import capabilities as _capabilities
 from openstack.block_storage.v3 import extension as _extension
 from openstack.block_storage.v3 import group as _group
@@ -21,6 +25,7 @@ from openstack.block_storage.v3 import group_type as _group_type
 from openstack.block_storage.v3 import limits as _limits
 from openstack.block_storage.v3 import quota_set as _quota_set
 from openstack.block_storage.v3 import resource_filter as _resource_filter
+from openstack.block_storage.v3 import service as _service
 from openstack.block_storage.v3 import snapshot as _snapshot
 from openstack.block_storage.v3 import stats as _stats
 from openstack.block_storage.v3 import type as _type
@@ -33,19 +38,21 @@ from openstack import resource
 class Proxy(_base_proxy.BaseBlockStorageProxy):
     _resource_registry = {
         "availability_zone": availability_zone.AvailabilityZone,
+        "attachment": _attachment.Attachment,
         "backup": _backup.Backup,
         "capabilities": _capabilities.Capabilities,
         "extension": _extension.Extension,
         "group": _group.Group,
         "group_snapshot": _group_snapshot.GroupSnapshot,
-        "group_type": _group_type,
+        "group_type": _group_type.GroupType,
         "limits": _limits.Limit,
         "quota_set": _quota_set.QuotaSet,
         "resource_filter": _resource_filter.ResourceFilter,
         "snapshot": _snapshot.Snapshot,
         "stats_pools": _stats.Pools,
+        "summary": _summary.BlockStorageSummary,
         "type": _type.Type,
-        "volume": _volume.Volume
+        "volume": _volume.Volume,
     }
 
     # ====== SNAPSHOTS ======
@@ -168,7 +175,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         if not force:
             self._delete(
-                _snapshot.Snapshot, snapshot, ignore_missing=ignore_missing)
+                _snapshot.Snapshot, snapshot, ignore_missing=ignore_missing
+            )
         else:
             snapshot = self._get_resource(_snapshot.Snapshot, snapshot)
             snapshot.force_delete(self)
@@ -405,9 +413,11 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         volume_type = self._get_resource(_type.Type, volume_type_id)
 
-        return self._get(_type.TypeEncryption,
-                         volume_type_id=volume_type.id,
-                         requires_id=False)
+        return self._get(
+            _type.TypeEncryption,
+            volume_type_id=volume_type.id,
+            requires_id=False,
+        )
 
     def create_type_encryption(self, volume_type, **attrs):
         """Create new type encryption from attributes
@@ -425,11 +435,13 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         volume_type = self._get_resource(_type.Type, volume_type)
 
-        return self._create(_type.TypeEncryption,
-                            volume_type_id=volume_type.id, **attrs)
+        return self._create(
+            _type.TypeEncryption, volume_type_id=volume_type.id, **attrs
+        )
 
-    def delete_type_encryption(self, encryption=None,
-                               volume_type=None, ignore_missing=True):
+    def delete_type_encryption(
+        self, encryption=None, volume_type=None, ignore_missing=True
+    ):
         """Delete type encryption attributes
 
         :param encryption: The value can be None or a
@@ -452,12 +464,15 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
 
         if volume_type:
             volume_type = self._get_resource(_type.Type, volume_type)
-            encryption = self._get(_type.TypeEncryption,
-                                   volume_type=volume_type.id,
-                                   requires_id=False)
+            encryption = self._get(
+                _type.TypeEncryption,
+                volume_type=volume_type.id,
+                requires_id=False,
+            )
 
-        self._delete(_type.TypeEncryption, encryption,
-                     ignore_missing=ignore_missing)
+        self._delete(
+            _type.TypeEncryption, encryption, ignore_missing=ignore_missing
+        )
 
     def update_type_encryption(
         self,
@@ -654,6 +669,26 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         else:
             volume.delete_metadata(self)
 
+    def summary(self, all_projects):
+        """Get Volumes Summary
+
+        This method returns the volumes summary in the deployment.
+
+        :param all_projects: Whether to return the summary of all projects
+            or not.
+
+        :returns: One :class:
+            `~openstack.block_storage.v3.block_storage_summary.Summary`
+            instance.
+        """
+        res = self._get(_summary.BlockStorageSummary, requires_id=False)
+        return res.fetch(
+            self,
+            requires_id=False,
+            resource_response_key='volume-summary',
+            all_projects=all_projects,
+        )
+
     # ====== VOLUME ACTIONS ======
     def extend_volume(self, volume, size):
         """Extend a volume
@@ -725,9 +760,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         volume = self._get_resource(_volume.Volume, volume)
         volume.reset_status(self, status, attach_status, migration_status)
 
-    def revert_volume_to_snapshot(
-        self, volume, snapshot
-    ):
+    def revert_volume_to_snapshot(self, volume, snapshot):
         """Revert a volume to its latest snapshot.
 
         This method only support reverting a detached volume, and the
@@ -744,9 +777,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         snapshot = self._get_resource(_snapshot.Snapshot, snapshot)
         volume.revert_to_snapshot(self, snapshot.id)
 
-    def attach_volume(
-        self, volume, mountpoint, instance=None, host_name=None
-    ):
+    def attach_volume(self, volume, mountpoint, instance=None, host_name=None):
         """Attaches a volume to a server.
 
         :param volume: The value can be either the ID of a volume or a
@@ -760,9 +791,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         volume = self._get_resource(_volume.Volume, volume)
         volume.attach(self, mountpoint, instance, host_name)
 
-    def detach_volume(
-        self, volume, attachment, force=False, connector=None
-    ):
+    def detach_volume(self, volume, attachment, force=False, connector=None):
         """Detaches a volume from a server.
 
         :param volume: The value can be either the ID of a volume or a
@@ -784,13 +813,17 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :param volume: The value can be either the ID of a volume or a
             :class:`~openstack.block_storage.v3.volume.Volume` instance.
 
-        :returns: None """
+        :returns: None"""
         volume = self._get_resource(_volume.Volume, volume)
         volume.unmanage(self)
 
     def migrate_volume(
-        self, volume, host=None, force_host_copy=False,
-        lock_volume=False, cluster=None
+        self,
+        volume,
+        host=None,
+        force_host_copy=False,
+        lock_volume=False,
+        cluster=None,
     ):
         """Migrates a volume to the specified host.
 
@@ -816,9 +849,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         volume = self._get_resource(_volume.Volume, volume)
         volume.migrate(self, host, force_host_copy, lock_volume, cluster)
 
-    def complete_volume_migration(
-        self, volume, new_volume, error=False
-    ):
+    def complete_volume_migration(self, volume, new_volume, error=False):
         """Complete the migration of a volume.
 
         :param volume: The value can be either the ID of a volume or a
@@ -833,8 +864,14 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         volume.complete_migration(self, new_volume, error)
 
     def upload_volume_to_image(
-        self, volume, image_name, force=False, disk_format=None,
-        container_format=None, visibility=None, protected=None
+        self,
+        volume,
+        image_name,
+        force=False,
+        disk_format=None,
+        container_format=None,
+        visibility=None,
+        protected=None,
     ):
         """Uploads the specified volume to image service.
 
@@ -851,10 +888,14 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :returns: dictionary describing the image.
         """
         volume = self._get_resource(_volume.Volume, volume)
-        volume.upload_to_image(
-            self, image_name, force=force, disk_format=disk_format,
-            container_format=container_format, visibility=visibility,
-            protected=protected
+        return volume.upload_to_image(
+            self,
+            image_name,
+            force=force,
+            disk_format=disk_format,
+            container_format=container_format,
+            visibility=visibility,
+            protected=protected,
         )
 
     def reserve_volume(self, volume):
@@ -863,7 +904,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :param volume: The value can be either the ID of a volume or a
             :class:`~openstack.block_storage.v3.volume.Volume` instance.
 
-        :returns: None """
+        :returns: None"""
         volume = self._get_resource(_volume.Volume, volume)
         volume.reserve(self)
 
@@ -873,7 +914,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :param volume: The value can be either the ID of a volume or a
             :class:`~openstack.block_storage.v3.volume.Volume` instance.
 
-        :returns: None """
+        :returns: None"""
         volume = self._get_resource(_volume.Volume, volume)
         volume.unreserve(self)
 
@@ -883,7 +924,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :param volume: The value can be either the ID of a volume or a
             :class:`~openstack.block_storage.v3.volume.Volume` instance.
 
-        :returns: None """
+        :returns: None"""
         volume = self._get_resource(_volume.Volume, volume)
         volume.begin_detaching(self)
 
@@ -893,7 +934,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :param volume: The value can be either the ID of a volume or a
             :class:`~openstack.block_storage.v3.volume.Volume` instance.
 
-        :returns: None """
+        :returns: None"""
         volume = self._get_resource(_volume.Volume, volume)
         volume.abort_detaching(self)
 
@@ -904,7 +945,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             :class:`~openstack.block_storage.v3.volume.Volume` instance.
         :param dict connector: The connector object.
 
-        :returns: None """
+        :returns: None"""
         volume = self._get_resource(_volume.Volume, volume)
         volume.init_attachment(self, connector)
 
@@ -920,6 +961,113 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         volume = self._get_resource(_volume.Volume, volume)
         volume.terminate_attachment(self, connector)
 
+    # ====== ATTACHMENTS ======
+
+    def create_attachment(self, volume, **attrs):
+        """Create a new attachment
+
+        This is an internal API and should only be called by services
+        consuming volume attachments like nova, glance, ironic etc.
+
+        :param volume: The value can be either the ID of a volume or a
+            :class:`~openstack.block_storage.v3.volume.Volume` instance.
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.block_storage.v3.attachment.Attachment`
+            comprised of the properties on the Attachment class like
+            connector, instance_id, mode etc.
+        :returns: The results of attachment creation
+        :rtype: :class:`~openstack.block_storage.v3.attachment.Attachment`
+        """
+        volume_id = resource.Resource._get_id(volume)
+        return self._create(
+            _attachment.Attachment, volume_id=volume_id, **attrs
+        )
+
+    def get_attachment(self, attachment):
+        """Get a single volume
+
+        This is an internal API and should only be called by services
+        consuming volume attachments like nova, glance, ironic etc.
+
+        :param attachment: The value can be the ID of an attachment or a
+            :class:`~attachment.Attachment` instance.
+
+        :returns: One :class:`~attachment.Attachment`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._get(_attachment.Attachment, attachment)
+
+    def attachments(self, **query):
+        """Returns a generator of attachments.
+
+        This is an internal API and should only be called by services
+        consuming volume attachments like nova, glance, ironic etc.
+
+        :param kwargs query: Optional query parameters to be sent to limit
+            the resources being returned.
+
+        :returns: A generator of attachment objects.
+        """
+        return self._list(_attachment.Attachment, **query)
+
+    def delete_attachment(self, attachment, ignore_missing=True):
+        """Delete an attachment
+
+        This is an internal API and should only be called by services
+        consuming volume attachments like nova, glance, ironic etc.
+
+        :param type: The value can be either the ID of a attachment or a
+            :class:`~openstack.block_storage.v3.attachment.Attachment`
+            instance.
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be
+            raised when the attachment does not exist.
+            When set to ``True``, no exception will be set when
+            attempting to delete a nonexistent attachment.
+
+        :returns: ``None``
+        """
+        self._delete(
+            _attachment.Attachment,
+            attachment,
+            ignore_missing=ignore_missing,
+        )
+
+    def update_attachment(self, attachment, **attrs):
+        """Update an attachment
+
+        This is an internal API and should only be called by services
+        consuming volume attachments like nova, glance, ironic etc.
+
+        :param attachment: The value can be the ID of an attachment or a
+            :class:`~openstack.block_storage.v3.attachment.Attachment`
+            instance.
+        :param dict attrs: Keyword arguments which will be used to update
+            a :class:`~openstack.block_storage.v3.attachment.Attachment`
+            comprised of the properties on the Attachment class
+
+        :returns: The updated attachment
+        :rtype: :class:`~openstack.volume.v3.attachment.Attachment`
+        """
+        return self._update(_attachment.Attachment, attachment, **attrs)
+
+    def complete_attachment(self, attachment):
+        """Complete an attachment
+
+        This is an internal API and should only be called by services
+        consuming volume attachments like nova, glance, ironic etc.
+
+        :param attachment: The value can be the ID of an attachment or a
+            :class:`~openstack.block_storage.v3.attachment.Attachment`
+            instance.
+
+        :returns: ``None``
+        :rtype: :class:`~openstack.volume.v3.attachment.Attachment`
+        """
+        attachment_obj = self._get_resource(_attachment.Attachment, attachment)
+        return attachment_obj.complete(self)
+
     # ====== BACKEND POOLS ======
     def backend_pools(self, **query):
         """Returns a generator of cinder Back-end storage pools
@@ -1022,8 +1170,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :returns: ``None``
         """
         if not force:
-            self._delete(
-                _backup.Backup, backup, ignore_missing=ignore_missing)
+            self._delete(_backup.Backup, backup, ignore_missing=ignore_missing)
         else:
             backup = self._get_resource(_backup.Backup, backup)
             backup.force_delete(self)
@@ -1056,16 +1203,23 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         backup.reset(self, status)
 
     # ====== LIMITS ======
-    def get_limits(self):
+    def get_limits(self, project=None):
         """Retrieves limits
 
+        :param project: A project to get limits for. The value can be either
+            the ID of a project or an
+            :class:`~openstack.identity.v3.project.Project` instance.
         :returns: A Limit object, including both
             :class:`~openstack.block_storage.v3.limits.AbsoluteLimit` and
             :class:`~openstack.block_storage.v3.limits.RateLimit`
         :rtype: :class:`~openstack.block_storage.v3.limits.Limit`
         """
-        return self._get(_limits.Limit, requires_id=False)
+        params = {}
+        if project:
+            params['project_id'] = resource.Resource._get_id(project)
+        return self._get(_limits.Limit, requires_id=False, **params)
 
+    # ====== CAPABILITIES ======
     def get_capabilities(self, host):
         """Get a backend's capabilites
 
@@ -1295,7 +1449,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :returns: None
         """
         resource = self._get_resource(
-            _group_snapshot.GroupSnapshot, group_snapshot)
+            _group_snapshot.GroupSnapshot, group_snapshot
+        )
         resource.reset_state(self, state)
 
     def delete_group_snapshot(self, group_snapshot, ignore_missing=True):
@@ -1307,8 +1462,10 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :returns: None
         """
         self._delete(
-            _group_snapshot.GroupSnapshot, group_snapshot,
-            ignore_missing=ignore_missing)
+            _group_snapshot.GroupSnapshot,
+            group_snapshot,
+            ignore_missing=ignore_missing,
+        )
 
     # ====== GROUP TYPE ======
     def get_group_type(self, group_type):
@@ -1395,7 +1552,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :returns: None
         """
         self._delete(
-            _group_type.GroupType, group_type, ignore_missing=ignore_missing)
+            _group_type.GroupType, group_type, ignore_missing=ignore_missing
+        )
 
     def update_group_type(self, group_type, **attrs):
         """Update a group_type
@@ -1408,8 +1566,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :returns: The updated group type.
         :rtype: :class:`~openstack.block_storage.v3.group_type.GroupType`
         """
-        return self._update(
-            _group_type.GroupType, group_type, **attrs)
+        return self._update(_group_type.GroupType, group_type, **attrs)
 
     def fetch_group_type_group_specs(self, group_type):
         """Lists group specs of a group type.
@@ -1488,9 +1645,9 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         project = self._get_resource(_project.Project, project)
         res = self._get_resource(
-            _quota_set.QuotaSet, None, project_id=project.id)
-        return res.fetch(
-            self, usage=usage, **query)
+            _quota_set.QuotaSet, None, project_id=project.id
+        )
+        return res.fetch(self, usage=usage, **query)
 
     def get_quota_set_defaults(self, project):
         """Show QuotaSet defaults for the project
@@ -1505,9 +1662,9 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         project = self._get_resource(_project.Project, project)
         res = self._get_resource(
-            _quota_set.QuotaSet, None, project_id=project.id)
-        return res.fetch(
-            self, base_path='/os-quota-sets/defaults')
+            _quota_set.QuotaSet, None, project_id=project.id
+        )
+        return res.fetch(self, base_path='/os-quota-sets/defaults')
 
     def revert_quota_set(self, project, **query):
         """Reset Quota for the project/user.
@@ -1521,7 +1678,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         project = self._get_resource(_project.Project, project)
         res = self._get_resource(
-            _quota_set.QuotaSet, None, project_id=project.id)
+            _quota_set.QuotaSet, None, project_id=project.id
+        )
 
         return res.delete(self, **query)
 
@@ -1542,6 +1700,150 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             query = {}
         return res.commit(self, **query)
 
+    # ====== SERVICES ======
+    @ty.overload
+    def find_service(
+        self,
+        name_or_id: str,
+        ignore_missing: ty.Literal[True] = True,
+        **query,
+    ) -> ty.Optional[_service.Service]:
+        ...
+
+    @ty.overload
+    def find_service(
+        self,
+        name_or_id: str,
+        ignore_missing: ty.Literal[False] = True,
+        **query,
+    ) -> _service.Service:
+        ...
+
+    def find_service(
+        self,
+        name_or_id: str,
+        ignore_missing: bool = True,
+        **query,
+    ) -> ty.Optional[_service.Service]:
+        """Find a single service
+
+        :param name_or_id: The name or ID of a service
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            the resource does not exist.
+            When set to ``True``, None will be returned when attempting to find
+            a nonexistent resource.
+        :param dict query: Additional attributes like 'host'
+
+        :returns: One: class:`~openstack.block_storage.v3.service.Service` or None
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        :raises: :class:`~openstack.exceptions.DuplicateResource` when multiple
+            resources are found.
+        """
+        return self._find(
+            _service.Service,
+            name_or_id,
+            ignore_missing=ignore_missing,
+            **query,
+        )
+
+    def services(
+        self,
+        **query: ty.Any,
+    ) -> ty.Generator[_service.Service, None, None]:
+        """Return a generator of service
+
+        :param kwargs query: Optional query parameters to be sent to limit
+            the resources being returned.
+        :returns: A generator of Service objects
+        :rtype: class: `~openstack.block_storage.v3.service.Service`
+        """
+        return self._list(_service.Service, **query)
+
+    def enable_service(
+        self,
+        service: ty.Union[str, _service.Service],
+    ) -> _service.Service:
+        """Enable a service
+
+        :param service: Either the ID of a service or a
+            :class:`~openstack.block_storage.v3.service.Service` instance.
+
+        :returns: Updated service instance
+        :rtype: class: `~openstack.block_storage.v3.service.Service`
+        """
+        service = self._get_resource(_service.Service, service)
+        return service.enable(self)
+
+    def disable_service(
+        self,
+        service: ty.Union[str, _service.Service],
+        *,
+        reason: ty.Optional[str] = None,
+    ) -> _service.Service:
+        """Disable a service
+
+        :param service: Either the ID of a service or a
+            :class:`~openstack.block_storage.v3.service.Service` instance
+        :param str reason: The reason to disable a service
+
+        :returns: Updated service instance
+        :rtype: class: `~openstack.block_storage.v3.service.Service`
+        """
+        service = self._get_resource(_service.Service, service)
+        return service.disable(self, reason=reason)
+
+    def thaw_service(
+        self,
+        service: ty.Union[str, _service.Service],
+    ) -> _service.Service:
+        """Thaw a service
+
+        :param service: Either the ID of a service or a
+            :class:`~openstack.block_storage.v3.service.Service` instance
+
+        :returns: Updated service instance
+        :rtype: class: `~openstack.block_storage.v3.service.Service`
+        """
+        service = self._get_resource(_service.Service, service)
+        return service.thaw(self)
+
+    def freeze_service(
+        self,
+        service: ty.Union[str, _service.Service],
+    ) -> _service.Service:
+        """Freeze a service
+
+        :param service: Either the ID of a service or a
+            :class:`~openstack.block_storage.v3.service.Service` instance
+
+        :returns: Updated service instance
+        :rtype: class: `~openstack.block_storage.v3.service.Service`
+        """
+        service = self._get_resource(_service.Service, service)
+        return service.freeze(self)
+
+    def failover_service(
+        self,
+        service: ty.Union[str, _service.Service],
+        *,
+        cluster: ty.Optional[str] = None,
+        backend_id: ty.Optional[str] = None,
+    ) -> _service.Service:
+        """Failover a service
+
+        Only applies to replicating cinder-volume services.
+
+        :param service: Either the ID of a service or a
+            :class:`~openstack.block_storage.v3.service.Service` instance
+
+        :returns: Updated service instance
+        :rtype: class: `~openstack.block_storage.v3.service.Service`
+        """
+        service = self._get_resource(_service.Service, service)
+        return service.failover(self, cluster=cluster, backend_id=backend_id)
+
     # ====== RESOURCE FILTERS ======
     def resource_filters(self, **query):
         """Retrieve a generator of resource filters
@@ -1561,7 +1863,13 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
 
     # ====== UTILS ======
     def wait_for_status(
-        self, res, status='available', failures=None, interval=2, wait=120,
+        self,
+        res,
+        status='available',
+        failures=None,
+        interval=2,
+        wait=120,
+        callback=None,
     ):
         """Wait for a resource to be in a particular status.
 
@@ -1574,6 +1882,9 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             checks. Default to 2.
         :param wait: Maximum number of seconds to wait before the change.
             Default to 120.
+        :param callback: A callback function. This will be called with a single
+            value, progress.
+
         :returns: The resource is returned on success.
         :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
             to the desired status failed to occur in specified seconds.
@@ -1584,9 +1895,16 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         failures = ['error'] if failures is None else failures
         return resource.wait_for_status(
-            self, res, status, failures, interval, wait)
+            self,
+            res,
+            status,
+            failures,
+            interval,
+            wait,
+            callback=callback,
+        )
 
-    def wait_for_delete(self, res, interval=2, wait=120):
+    def wait_for_delete(self, res, interval=2, wait=120, callback=None):
         """Wait for a resource to be deleted.
 
         :param res: The resource to wait on to be deleted.
@@ -1595,18 +1913,23 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             checks. Default to 2.
         :param int wait: Maximum number of seconds to wait before the change.
             Default to 120.
+        :param callback: A callback function. This will be called with a single
+            value, progress.
+
         :returns: The resource is returned on success.
         :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
             to delete failed to occur in the specified seconds.
         """
-        return resource.wait_for_delete(self, res, interval, wait)
+        return resource.wait_for_delete(
+            self,
+            res,
+            interval,
+            wait,
+            callback=callback,
+        )
 
     def _get_cleanup_dependencies(self):
-        return {
-            'block_storage': {
-                'before': []
-            }
-        }
+        return {'block_storage': {'before': []}}
 
     def _service_cleanup(
         self,
@@ -1614,7 +1937,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         client_status_queue=None,
         identified_resources=None,
         filters=None,
-        resource_evaluation_fn=None
+        resource_evaluation_fn=None,
+        skip_resources=None,
     ):
         # It is not possible to delete backup if there are dependent backups.
         # In order to be able to do cleanup those is required to have multiple
@@ -1624,82 +1948,89 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         # required to limit amount of iterations we do (currently pick 10).  In
         # dry_run all those iterations are doing not what we want, therefore
         # only iterate in a real cleanup mode.
-        if dry_run:
-            # Just iterate and evaluate backups in dry_run mode
-            for obj in self.backups(details=False):
+        if not self.should_skip_resource_cleanup("backup", skip_resources):
+            if dry_run:
+                # Just iterate and evaluate backups in dry_run mode
+                for obj in self.backups(details=False):
+                    need_delete = self._service_cleanup_del_res(
+                        self.delete_backup,
+                        obj,
+                        dry_run=dry_run,
+                        client_status_queue=client_status_queue,
+                        identified_resources=identified_resources,
+                        filters=filters,
+                        resource_evaluation_fn=resource_evaluation_fn,
+                    )
+            else:
+                # Set initial iterations conditions
+                need_backup_iteration = True
+                max_iterations = 10
+                while need_backup_iteration and max_iterations > 0:
+                    # Reset iteration controls
+                    need_backup_iteration = False
+                    max_iterations -= 1
+                    backups = []
+                    # To increase success chance sort backups by age, dependent
+                    # backups are logically younger.
+                    for obj in self.backups(
+                        details=True, sort_key='created_at', sort_dir='desc'
+                    ):
+                        if not obj.has_dependent_backups:
+                            # If no dependent backups - go with it
+                            need_delete = self._service_cleanup_del_res(
+                                self.delete_backup,
+                                obj,
+                                dry_run=dry_run,
+                                client_status_queue=client_status_queue,
+                                identified_resources=identified_resources,
+                                filters=filters,
+                                resource_evaluation_fn=resource_evaluation_fn,
+                            )
+                            if not dry_run and need_delete:
+                                backups.append(obj)
+                        else:
+                            # Otherwise we need another iteration
+                            need_backup_iteration = True
+
+                    # Before proceeding need to wait for backups to be deleted
+                    for obj in backups:
+                        try:
+                            self.wait_for_delete(obj)
+                        except exceptions.SDKException:
+                            # Well, did our best, still try further
+                            pass
+
+        if not self.should_skip_resource_cleanup("snapshot", skip_resources):
+            snapshots = []
+            for obj in self.snapshots(details=False):
                 need_delete = self._service_cleanup_del_res(
-                    self.delete_backup,
+                    self.delete_snapshot,
                     obj,
                     dry_run=dry_run,
                     client_status_queue=client_status_queue,
                     identified_resources=identified_resources,
                     filters=filters,
-                    resource_evaluation_fn=resource_evaluation_fn)
-        else:
-            # Set initial iterations conditions
-            need_backup_iteration = True
-            max_iterations = 10
-            while need_backup_iteration and max_iterations > 0:
-                # Reset iteration controls
-                need_backup_iteration = False
-                max_iterations -= 1
-                backups = []
-                # To increase success chance sort backups by age, dependent
-                # backups are logically younger.
-                for obj in self.backups(
-                        details=True, sort_key='created_at', sort_dir='desc'
-                ):
-                    if not obj.has_dependent_backups:
-                        # If no dependent backups - go with it
-                        need_delete = self._service_cleanup_del_res(
-                            self.delete_backup,
-                            obj,
-                            dry_run=dry_run,
-                            client_status_queue=client_status_queue,
-                            identified_resources=identified_resources,
-                            filters=filters,
-                            resource_evaluation_fn=resource_evaluation_fn)
-                        if not dry_run and need_delete:
-                            backups.append(obj)
-                    else:
-                        # Otherwise we need another iteration
-                        need_backup_iteration = True
+                    resource_evaluation_fn=resource_evaluation_fn,
+                )
+                if not dry_run and need_delete:
+                    snapshots.append(obj)
 
-                # Before proceeding need to wait for backups to be deleted
-                for obj in backups:
-                    try:
-                        self.wait_for_delete(obj)
-                    except exceptions.SDKException:
-                        # Well, did our best, still try further
-                        pass
+            # Before deleting volumes need to wait for snapshots to be deleted
+            for obj in snapshots:
+                try:
+                    self.wait_for_delete(obj)
+                except exceptions.SDKException:
+                    # Well, did our best, still try further
+                    pass
 
-        snapshots = []
-        for obj in self.snapshots(details=False):
-            need_delete = self._service_cleanup_del_res(
-                self.delete_snapshot,
-                obj,
-                dry_run=dry_run,
-                client_status_queue=client_status_queue,
-                identified_resources=identified_resources,
-                filters=filters,
-                resource_evaluation_fn=resource_evaluation_fn)
-            if not dry_run and need_delete:
-                snapshots.append(obj)
-
-        # Before deleting volumes need to wait for snapshots to be deleted
-        for obj in snapshots:
-            try:
-                self.wait_for_delete(obj)
-            except exceptions.SDKException:
-                # Well, did our best, still try further
-                pass
-
-        for obj in self.volumes(details=True):
-            self._service_cleanup_del_res(
-                self.delete_volume,
-                obj,
-                dry_run=dry_run,
-                client_status_queue=client_status_queue,
-                identified_resources=identified_resources,
-                filters=filters,
-                resource_evaluation_fn=resource_evaluation_fn)
+        if not self.should_skip_resource_cleanup("volume", skip_resources):
+            for obj in self.volumes(details=True):
+                self._service_cleanup_del_res(
+                    self.delete_volume,
+                    obj,
+                    dry_run=dry_run,
+                    client_status_queue=client_status_queue,
+                    identified_resources=identified_resources,
+                    filters=filters,
+                    resource_evaluation_fn=resource_evaluation_fn,
+                )
